@@ -1,5 +1,6 @@
-import os
 from yafowil.base import factory
+from yafowil.utils import entry_point
+import os
 
 
 resourcedir = os.path.join(os.path.dirname(__file__), 'resources')
@@ -18,11 +19,19 @@ bootstrap_css = [{
     'resource': 'bootstrap/widget.css',
     'order': 20,
 }]
+plone5_css = [{
+    'group': 'yafowil.widget.array.common',
+    'resource': 'plone5/widget.css',
+    'order': 20,
+}]
 
 
+@entry_point(order=10)
 def register():
-    import widget
+    from yafowil.widget.array import widget  # noqa
     factory.register_theme('default', 'yafowil.widget.array',
                            resourcedir, js=js, css=default_css)
     factory.register_theme('bootstrap', 'yafowil.widget.array',
-                           resourcedir, js=js, css=bootstrap_css)
\ No newline at end of file
+                           resourcedir, js=js, css=bootstrap_css)
+    factory.register_theme('plone5', 'yafowil.widget.array',
+                           resourcedir, js=js, css=plone5_css)
