@@ -1,21 +1,15 @@
-import types
-from yafowil.base import (
-    UNSET,
-    factory,
-    fetch_value,
-)
-from yafowil.utils import (
-    cssid,
-    cssclasses,
-    css_managed_props,
-    managedprops,
-    attr_value,
-)
-from yafowil.common import (
-    generic_extractor,
-    generic_required_extractor,
-)
+from node.utils import UNSET
+from yafowil.base import factory
+from yafowil.base import fetch_value
+from yafowil.common import generic_extractor
+from yafowil.common import generic_required_extractor
+from yafowil.compat import ITER_TYPES
 from yafowil.compound import compound_renderer
+from yafowil.utils import attr_value
+from yafowil.utils import css_managed_props
+from yafowil.utils import cssclasses
+from yafowil.utils import cssid
+from yafowil.utils import managedprops
 
 
 @managedprops(*css_managed_props)
@@ -34,10 +28,10 @@ def array_display_proxy_renderer(widget, data):
 
 
 factory.register(
-     'array_display_proxy',
-     extractors=[generic_extractor],
-     edit_renderers=[array_display_proxy_renderer],
-     display_renderers=[array_display_proxy_renderer])
+    'array_display_proxy',
+    extractors=[generic_extractor],
+    edit_renderers=[array_display_proxy_renderer],
+    display_renderers=[array_display_proxy_renderer])
 
 # do not document internal widget
 factory.doc['blueprint']['array_display_proxy'] = UNSET
@@ -50,6 +44,7 @@ ICON_CSS = {
     'down': 'icon-circle-arrow-down',
 }
 
+
 def actions_renderer(widget, data):
     tag = data.tag
     actions = list()
@@ -114,7 +109,7 @@ def array_wrapper_renderer(widget, data):
         'id': cssid(widget, 'array'),
         'class': cssclasses(widget, data) + add_css,
     }
-    if not 'array' in kw['class'].split(' '):
+    if 'array' not in kw['class'].split(' '):
         kw['class'] = 'array %s' % kw['class']
     rendered = data.rendered
     return data.tag('div', rendered, **kw)
@@ -125,9 +120,9 @@ TEMPLATE = 'TEMPLATE'
 
 
 def array_edit_renderer(widget, data):
-    if len(widget) == 1 and not widget.has_key(CONTAINER):
+    if len(widget) == 1 and CONTAINER not in widget:
         raise Exception(u"Empty array widget defined")
-    if not widget.has_key(CONTAINER):
+    if CONTAINER not in widget:
         template = widget.detach(widget.keys()[1])
         hook_array_template(widget, template)
     value = fetch_value(widget, data)
@@ -145,18 +140,19 @@ def create_array_children(widget, template, value):
     if isinstance(value, dict):
         try:
             indices = [int(_) for _ in value.keys()]
-        except ValueError, e:
+        except ValueError as e:
             raise Exception(u"Array value error. %s" % str(e))
         indices = sorted(indices)
         for i in indices:
             create_array_entry(str(i), widget, template, value[str(i)])
-    elif type(value) in [types.ListType, types.TupleType]:
+    elif type(value) in ITER_TYPES:
         indices = range(len(value))
         for i in indices:
             create_array_entry(str(i), widget, template, value[i])
     else:
-        raise ValueError(u"Expected list or dict as value. Got '%s'" % \
-                         str(type(value)))
+        raise ValueError(u"Expected list or dict as value. Got '{}'".format(
+            str(type(value)))
+        )
 
 
 def create_array_entry(idx, widget, template, value):
@@ -188,10 +184,10 @@ def create_array_entry(idx, widget, template, value):
         orgin = template[template.keys()[-1]]
         template = duplicate_recursiv(orgin)
         hook_array_template(child_widget, template)
-        # If array in array with compound. Compound template children get 
+        # If array in array with compound. Compound template children get
         # hooked to array widget. Remove them.
         for key in child_widget.keys():
-            if not key in ['table', CONTAINER]:
+            if key not in ['table', CONTAINER]:
                 del child_widget[key]
         return
     create_array_entry_children(child_widget, template)
@@ -209,8 +205,8 @@ def hook_array_template(widget, template):
     props['class'] = 'arraytemplate'
     container = widget[CONTAINER] = factory('div', props=props)
     if template.attrs.get('structural'):
-        raise Exception(u"Compound templates for arrays must not be "
-                        u"structural.")
+        raise Exception(
+            u"Compound templates for arrays must not be structural.")
     container[TEMPLATE] = template
 
 
@@ -219,7 +215,7 @@ def duplicate_widget(widget, value=UNSET):
     leaf = len(widget) == 0
     # B/C: use display_property widget attribute!
     if leaf and (widget.mode == 'display' or widget.attrs.get('disabled')) \
-      and not 'array_display_proxy' in blueprints:
+            and 'array_display_proxy' not in blueprints:
         if not widget.attrs.get('display_proxy'):
             blueprints.insert(0, 'array_display_proxy')
     return factory(
@@ -263,11 +259,19 @@ def check_base_name_in_request(widget, request):
     return False
 
 
+def set_display_mode(widget):
+    for child in widget.values():
+        child.mode = 'display'
+        set_display_mode(child)
+
+
 def array_display_renderer(widget, data):
-    """XXX
-    """
-    raise NotImplementedError(u"yafowil.widget.array: Display mode not "
-                              u"implemented yet")
+    array_edit_renderer(widget, data)
+    del widget['table']['head']['row']['actions']
+    for row in widget['table']['body'].values():
+        del row['actions_col']
+    del widget[CONTAINER]
+    set_display_mode(widget)
 
 
 factory.register(
@@ -278,9 +282,9 @@ factory.register(
     display_renderers=[array_display_renderer, compound_renderer],
     builders=[array_builder])
 
-factory.doc['blueprint']['array'] = \
-"""Add-on widget `yafowil.widget.array 
-<http://github.com/bluedynamics/yafowil.widget.array/>`_.
+factory.doc['blueprint']['array'] = """\
+Add-on widget `yafowil.widget.array
+<http://github.com/conestack/yafowil.widget.array/>`_.
 """
 
 factory.defaults['array.class'] = 'array'
@@ -292,27 +296,27 @@ factory.defaults['array.error_class'] = 'error'
 factory.defaults['array.message_class'] = 'errormessage'
 
 factory.defaults['array.table_class'] = None
-factory.doc['props']['array.table_class'] = \
-"""CSS classes rendered on array table.
+factory.doc['props']['array.table_class'] = """\
+CSS classes rendered on array table.
 """
 
 factory.defaults['array.static'] = False
-factory.doc['props']['array.static'] = \
-"""Array is immutable. No Array actions are rendered. If True, ``add``,
+factory.doc['props']['array.static'] = """\
+Array is immutable. No Array actions are rendered. If True, ``add``,
 ``remove`` and ``sort`` properties are ignored.
 """
 
 factory.defaults['array.add'] = True
-factory.doc['props']['array.add'] = \
-"""Render ``add`` action.
+factory.doc['props']['array.add'] = """\
+Render ``add`` action.
 """
 
 factory.defaults['array.remove'] = True
-factory.doc['props']['array.remove'] = \
-"""Render ``remove`` action.
+factory.doc['props']['array.remove'] = """\
+Render ``remove`` action.
 """
 
 factory.defaults['array.sort'] = True
-factory.doc['props']['array.sort'] = \
-"""Render ``move up`` and ``move down`` actions.
+factory.doc['props']['array.sort'] = """\
+Render ``move up`` and ``move down`` actions.
 """
